##############################################################################
#    pvm
#    software development environment
#
#    Copyright (C) 1997  Andrew Guryanov
#    andrew-guryanov@usa.net
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
##############################################################################

#===========================================
#	Global variables and initialization
#===========================================

bind Entry <Control-x>   {tk_textCut %W}
bind Entry <Control-X>   {tk_textCut %W}
bind Entry <Control-c>   {tk_textCopy %W}
bind Entry <Control-C>   {tk_textCopy %W}
bind Entry <Control-v>   {tk_textPaste %W}
bind Entry <Control-V>   {tk_textPaste %W}

main:RegisterDocType "doc-text"
bind Text <Control-v>   {}
bind Text <Control-f>   {main:OnEditFind}
bind Text <Control-F>   {main:OnEditFind}
bind Text <Control-x>   {main:DocDispatch OnEditCut}
bind Text <Control-X>   {main:DocDispatch OnEditCut}
bind Text <Control-c>   {main:DocDispatch OnEditCopy}
bind Text <Control-C>   {main:DocDispatch OnEditCopy}
bind Text <Control-v>   {main:DocDispatch OnEditPaste}
bind Text <Control-V>   {main:DocDispatch OnEditPaste}
bind Text <Control-g>   {main:DocDispatch OnEditGoto}
bind Text <Control-G>   {main:DocDispatch OnEditGoto}
bind Text <Control-F3>  {main:DocDispatch OnEditFindSelected}
bind Text <F3>          {main:DocDispatch OnEditFindAgain}

bind Text <Delete>      {main:DocDispatch OnEditDelete}
bind Text <BackSpace>   {main:DocDispatch OnEditBackspace}
bind Text <KeyPress>    {main:DocDispatch OnKeyPress "" %K %A}
bind Text <KeyRelease>  {+main:DocDispatch ShowPosition}



#===========================================
#	Text document
#===========================================


proc doc-text:MatchDocType {filename} {
#---------------------------------------------------
    return 5
}

proc doc-text:OnCreateClient {this} {
#---------------------------------
    global docdata
    set id [main:GetWindowId $this]
    set docdata($id,lastfind) ""

    text $this.text -relief sunken  -borderwidth 1 -wrap none -tabs 0.7c -width 1 -height 1 \
        -yscrollcommand "$this.yscroll set" -xscrollcommand "$this.xscroll set"
    scrollbar $this.yscroll -orient vertical -relief ridge -command "$this.text yview"
    scrollbar $this.xscroll -orient horizontal -relief ridge -command "$this.text xview"
    pack $this.yscroll -side right -fill y
    pack $this.xscroll -side bottom -fill x
    pack $this.text -expand 1 -fill both -side bottom

    bind $this.text <ButtonRelease-1>    {+main:DocDispatch ShowPosition}
    bind $this.text <FocusIn>     "+main:DocDispatch ShowPosition $this"
    bind $this.text <FocusOut>    {+main:SetStatusBarText "" 1}
}

proc doc-text:OnOpenDocument {this filename} {
#--------------------------------------------
    doc:OnOpenDocument $this $filename
    set f [open $filename "r"]
    if {$f != ""} {
        while {![eof $f]} {
            $this.text insert end [read $f 1000]
        }
        close $f
        $this.text mark set insert 1.0
        $this.text see insert
    }
}

proc doc-text:OnSaveDocument {this filename} {
#--------------------------------------------
    set f [open $filename "w"]
    if {$f != ""} {
        for {set istart 1.0; set iend [$this.text index "$istart+1024c"]; set iend [$this.text index "$iend-1c"]} \
            {$istart < $iend} \
            {set istart $iend; set iend [$this.text index "$istart+1024c"]; set iend [$this.text index "$iend-1c"]} {
            puts -nonewline $f [$this.text get $istart $iend]
        }
        doc:SetModified $this false
        close $f
    }
}

proc doc-text:OnCloseDocument {this} {
#--------------------------------------
    main:SetStatusBarText "" 1
    doc:OnCloseDocument $this
}

proc doc-text:OnChangeAppearance {this} {
#-----------------------------------------
    $this.text configure \
        -font [main:GetMainData font def] \
        -fg [main:GetMainData color textfg] \
        -bg [main:GetMainData color textbg] \
        -selectforeground [main:GetMainData color textselfg] \
        -selectbackground [main:GetMainData color textselbg] \
        -insertbackground [main:GetMainData color textfg]
}

proc doc-text:OnEditCut {this} {
#-----------------------------------------
    tk_textCut $this.text
    $this.text see insert
    doc:SetModified $this
}

proc doc-text:OnEditCopy {this} {
#-----------------------------------------
    tk_textCopy $this.text
}

proc doc-text:OnEditPaste {this} {
#-----------------------------------------
    tk_textPaste $this.text
    $this.text see insert
    doc:SetModified $this
}

proc doc-text:OnEditDelete {this} {
#-----------------------------------------
    if {[$this.text tag nextrange sel 1.0 end] != ""} {

#        set tmp [$this.text get sel.first sel.last]
#        set pos [$this.text index sel.first]
#        puts "remove = $tmp = at = $pos"

        $this.text delete sel.first sel.last
    } else {

#        set tmp [$this.text get insert]
#        set pos [$this.text index insert]
#        puts "remove = $tmp = at = $pos"

        $this.text delete insert
        $this.text see insert
    }
    doc:SetModified $this
}

proc doc-text:OnEditBackspace {this} {
#-----------------------------------------
    if {[$this.text tag nextrange sel 1.0 end] != ""} {

#        set tmp [$this.text get sel.first sel.last]
#        set pos [$this.text index sel.first]
#        puts "remove = $tmp = at = $pos"

        $this.text delete sel.first sel.last
    } elseif [$this.text compare insert != 1.0] {

#        set tmp [$this.text get insert-1c]
#        set pos [$this.text index insert-1c]
#        puts "remove = $tmp = at = $pos"

        $this.text delete insert-1c
        $this.text see insert
    }
    doc:SetModified $this
}

proc doc-text:OnEditSelectAll {this} {
#-----------------------------------------
    set iend [$this.text index end-1c]
    $this.text tag remove 1.0 $iend
	$this.text tag add sel 1.0 $iend
#    $this.text mark set insert $iend
#    $this.text see insert
    $this.text mark set anchor insert
#    doc-text:ShowPosition $this
}

proc doc-text:OnEditGoto {this} {
#-----------------------------------------
    set dlg [dialog-entry:CreateDialog $this "Go to" "Line number"]
    set res [main:DlgDispatch DoModal $dlg]
    if {$res != ""} {
        $this.text tag remove sel 0.0 end
        $this.text mark set insert ${res}.0
        $this.text mark set anchor insert
        $this.text see insert
    }
}

proc doc-text:OnEditFindSelected {this} {
#-----------------------------------------
    if {[$this.text tag nextrange sel 1.0 end] != ""} {
        set find [$this.text get sel.first sel.last]
        if {![doc-text:OnFindText $this $find]} {
            bell
        }
    }
}

proc doc-text:OnEditFindAgain {this} {
#-----------------------------------------
    global docdata
    set id [main:GetWindowId $this]
    if {$docdata($id,lastfind) != ""} {
        if {![doc-text:OnFindText $this $docdata($id,lastfind)]} {
            bell
        }
    }
}

proc doc-text:OnKeyPress {this key ascii} {
#--------------------------------------------
    if {$ascii != "" && $key != "Escape" && $key != "Delete"} {
        tkTextInsert $this.text $ascii
        doc:SetModified $this
    }
    doc-text:ShowPosition $this
}

proc doc-text:ShowPosition {this} {
#-----------------------------------------
	set pos [$this.text index insert]
	scan $pos %d.%d line col
	incr col
    main:SetStatusBarText [format "%d, %d" $line $col] 1
}

proc doc-text:OnFindText {this find {direction next} {case 0} {circle 1} {regexp 0}} {
#------------------------------------------------------------------------------------
    global docdata
    if {$regexp} {
        set mode "-regexp"
    } else {
        set mode "-exact"
    }
    if {$direction == "next"} {
        set dir "-forwards"
        set pos [$this.text index insert+1c]
        set endpos end
    } else {
        set dir "-backwards"
        set pos [$this.text index insert-1c]
        set endpos 1.0
    }
    if {$case} {
        if {$circle} {
            set first [$this.text search $dir $mode -count cnt -- "$find" $pos]
        } else {
            set first [$this.text search $dir $mode -count cnt -- "$find" $pos $endpos]
        }
    } else {
        if {$circle} {
            set first [$this.text search $dir $mode -nocase -count cnt -- "$find" $pos]
        } else {
            set first [$this.text search $dir $mode -nocase -count cnt -- "$find" $pos $endpos]
        }
    }
    if {$first != ""} {
        set last "${first}+${cnt}c"
        $this.text tag remove sel 0.0 $first
        $this.text tag add sel $first $last
        $this.text tag remove sel $last end
        $this.text mark set insert $first
        $this.text mark set anchor $last
        $this.text see insert
        set id [main:GetWindowId $this]
        set docdata($id,lastfind) "$find"
        return 1
    }
    return 0
}

proc doc-text:OnReplaceText {this replace} {
#----------------------------------------------------------
    main:DocDispatch OnEditDelete $this
    set first [$this.text index insert]
    set last  "${first}+[string length $replace]c"
    $this.text insert insert "$replace"

    $this.text tag remove sel 0.0 $first
    $this.text tag add sel $first $last
    $this.text tag remove sel $last end
    $this.text mark set insert $first
    $this.text see insert
}
